Package: iterators
Type: Package
Title: Provides Iterator Construct
Version: 1.0.14
Authors@R: c(person("Folashade", "Daniel", role="cre", email="fdaniel@microsoft.com"),
             person("Revolution", "Analytics", role=c("aut", "cph")),
             person("Steve", "Weston", role="aut"))
Description: Support for iterators, which allow a programmer to traverse
    through all the elements of a vector, list, or other collection of data.
URL: https://github.com/RevolutionAnalytics/iterators
Depends: R (>= 2.5.0), utils
Suggests: RUnit, foreach
License: Apache License (== 2.0)
NeedsCompilation: no
Packaged: 2022-01-16 18:19:31 UTC; folashade
Author: Folashade Daniel [cre],
  Revolution Analytics [aut, cph],
  Steve Weston [aut]
Maintainer: Folashade Daniel <fdaniel@microsoft.com>
Repository: CRAN
Date/Publication: 2022-02-05 00:50:08 UTC
