export(iter, nextElem, isplit)
export(irnorm, irunif, irbinom, irnbinom, irpois)
export(icount, idiv, ireadLines, iread.table)
export(makeIwrapper, isample)
export(icountn, iapply)
S3method("iter", "default")
S3method("iter", "iter")
S3method("iter", "matrix")
S3method("iter", "data.frame")
S3method("iter", "function")
S3method("nextElem", "containeriter")
S3method("nextElem", "matrixiter")
S3method("nextElem", "dataframeiter")
S3method("nextElem", "funiter")
S3method("nextElem", "abstractiter")
S3method("as.list", "iter")
S3method("isplit", "default")
S3method("isplit", "data.frame")
