#
# Copyright (c) 2008-2010 Revolution Analytics
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

iapply <- function(X, MARGIN) {
  xit <- icountn(dim(X)[MARGIN])

  nextEl <- function() {
    i <- nextElem(xit)
    j <- rep('', length(dim(X)))
    j[MARGIN] <- as.character(i)
    s <- paste('X[', paste(j, collapse=','), ']', sep='')
    x <- parse(text=s)
    eval(x)
  }

  it <- list(nextElem=nextEl)
  class(it) <- c('abstractiter', 'iter')
  it
}
